// The package, naturally.
package WDYSDecrypt;

// Import IOException.
import java.io.IOException;

/**
 * This class controls the decryption process. It makes sure the file is read in and
 * fed to the decrypting object, and makes sure the decrypted file is written
 * afterwards.
 * 
 * @author Berend "Kirk" Wouda
 * @version 1.01
 * @since 1.00
 * @see WDYSDecrypt.WDYSDecrypter
 * @see WDYSDecrypt.WDYSEncryptedFileReader
 * @see WDYSDecrypt.WDYSDecryptedFileWriter
 */
public class WDYSDecryptControl {
	/**
	 * Constructs a new decryption control object.
	 */
	public WDYSDecryptControl() {
		// Yay.
	}
	
	/**
	 * Decrypts the given file.
	 * 
	 * @param filename1 The name of the file to be read from (and decrypted).
	 * @param filename2 The name of the file to be written to.
	 */
	public void decrypt(String filename1, String filename2) {
		try {
			// Construct a file reader.
			WDYSEncryptedFileReader reader = new WDYSEncryptedFileReader(filename1);
			
			// Construct a file writer.
			WDYSDecryptedFileWriter writer = new WDYSDecryptedFileWriter(filename2, reader.getDecryptedDataLength());
			
			// Construct a decrypter.
			WDYSDecrypter decrypter = new WDYSDecrypter();
			
			// Continually ask the filereader for 2 times 4 bytes, give them to the
			// decrypter to decrypt it, and write the results to the new file, until
			// we run out of bytes.
			for(int index = 0; index < reader.getEncryptedDataLength(); index += 8) {
				// Read 8 bytes and feed them to the decrypter.
				writer.write8DecryptedDataBytes(decrypter.decryptQuad(reader.read4EncryptedDataBytes(), reader.read4EncryptedDataBytes()));
			}
			
			// Close the reader.
			reader.close();
			
			// Close the writer.
			writer.close();
			
			// Please the user.
			System.out.println(filename1 + " succesfully decrypted into " + filename2 + ".");
		}
		catch(IOException e) {
			// Alert the user.
			System.out.println("An error occured while trying to read the file: " + e.getMessage());
		}
		catch(WDYSDecryptException e) {
			// Alert the user.
			System.out.println("An error occured while decrypting the file: " + e.getMessage());
		}
	}
}