// The package, d'oh.
package WDYSDecrypt;

// Import the IO classes.
import java.io.*;

/**
 * This class writes decrypted WDYS files.
 * 
 * @author Berend "Kirk" Wouda
 * @version 1.01
 * @since 1.00
 */
public class WDYSDecryptedFileWriter {
	/**
	 * Constructs a new file writer around the given file, with the given length.
	 * The file will be cut off at the given length, and any more calls to write
	 * cause exceptions. This is to ensure that the padding data to make the decrypted
	 * file an multiple of 8 bytes, is not written to the file.
	 * 
	 * @param filename The name of the encrypted file.
	 * @param length The length this file will be.
	 */
	public WDYSDecryptedFileWriter(String filename, long length) throws IOException {
		// Construct a new BufferedOutputStream around this baby.
		// Well... Put in a FileInputStream just to be certain ;^)
		writer = new BufferedOutputStream(new FileOutputStream(filename));
		
		// Store the length.
		this.length = length;
	}
	
	
	/**
	 * Closes the file.
	 * 
	 * @throws IOException When IO stuff goes wrong.
	 */
	public void close() throws IOException {
		// Yeah baby!
		writer.close();
	}
	
	/**
	 * Writes the next 8 bytes to the file, until the length is reached.
	 * 
	 * @param octet The next 8 bytes to be written.
	 * @throws IOException When IO stuff goes wrong.
	 * @throws WDYSDecryptException When the indicated length is exceeded.
	 */
	public void write8DecryptedDataBytes(byte[] octet) throws IOException, WDYSDecryptException {
		// Check if we're near the end.
		if(written <= length - 8) {
			// Read. I mean write.
			writer.write(octet, 0, 8);
			
			// Increase the amount of bytes written.
			written = written + 8;
		}
		// Check if we still have bytes to write.
		else if(written <= length) {
			// We need to write as many bytes are there are left until length.
			int left = (int) (length - written);
			
			// Read. I mean write, dammit.
			writer.write(octet, 0, left);
			
			// Increase the amount of bytes written.
			written = written + left;
		}
		else {
			// We're done. If we get here writing is attemptd while the file is done.
			throw new WDYSDecryptException("Indicated file length exceeded. Possibly a corrupted WDYS file.");
		}
	}
	
	
	/**
	 * The length of the decrypted data chunk (stored seperately in the file).
	 */
	protected long length;

	/**
	 * The amount of bytes written.
	 */
	protected long written = 0;
		
	/**
	 * The writer. Well it's a stream actually.
	 */
	protected BufferedOutputStream writer;
}